@@ -2,7 +2,7 @@
 
 ////////////////////////////////////////////////////////////////////////
 //
-// Copyright (C) 2016-2023 The Octave Project Developers
+// Copyright (C) 2016-2024 The Octave Project Developers
 //
 // See the file COPYRIGHT.md in the top-level directory of this
 // distribution or <https://octave.org/copyright/>.
@@ -32,7 +32,7 @@
 #include "liboctinterp-build-info.h"
 
 std::string
-liboctinterp_hg_id (void)
+liboctinterp_hg_id ()
 {
   return "%OCTAVE_HG_ID%";
 }
