@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////
 //
-// Copyright (C) 1996-2023 The Octave Project Developers
+// Copyright (C) 1996-2024 The Octave Project Developers
 //
 // See the file COPYRIGHT.md in the top-level directory of this
 // distribution or <https://octave.org/copyright/>.
@@ -692,7 +692,7 @@ DASPK::integrate (const ColumnVector& tout, Matrix& xdot_out,
 }
 
 std::string
-DASPK::error_message (void) const
+DASPK::error_message () const
 {
   std::string retval;
 
