@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////
 //
-// Copyright (C) 2012-2022 The Octave Project Developers
+// Copyright (C) 2012-2024 The Octave Project Developers
 //
 // See the file COPYRIGHT.md in the top-level directory of this
 // distribution or <https://octave.org/copyright/>.
@@ -32,7 +32,7 @@
 
 singleton_cleanup_list *singleton_cleanup_list::s_instance = nullptr;
 
-singleton_cleanup_list::~singleton_cleanup_list (void)
+singleton_cleanup_list::~singleton_cleanup_list ()
 {
   for (fptr fcn : m_fcn_list)
     {
@@ -41,7 +41,7 @@ singleton_cleanup_list::~singleton_cleanup_list (void)
 }
 
 bool
-singleton_cleanup_list::instance_ok (void)
+singleton_cleanup_list::instance_ok ()
 {
   bool retval = true;
 
