@@ -1,4 +1,9 @@
-## Copyright (C) 2012-2019 pdiribarne
+########################################################################
+##
+## Copyright (C) 2012-2024 The Octave Project Developers
+##
+## See the file COPYRIGHT.md in the top-level directory of this
+## distribution or <https://octave.org/copyright/>.
 ##
 ## This file is part of Octave.
 ##
@@ -15,6 +20,8 @@
 ## You should have received a copy of the GNU General Public License
 ## along with Octave; see the file COPYING.  If not, see
 ## <https://www.gnu.org/licenses/>.
+##
+########################################################################
 
 ## -*- texinfo -*-
 ## @deftypefn  {} {@var{h} =} struct2hdl (@var{s})
@@ -28,7 +35,7 @@
 ## If the handle of an existing figure or axes is specified, @var{p}, the new
 ## object will be created as a child of that object.  If no parent handle is
 ## provided then a new figure and the necessary children will be constructed
-## using the default values from the root figure.
+## using the default values from the root object.
 ##
 ## A third boolean argument @var{hilev} can be passed to specify whether the
 ## function should preserve listeners/callbacks, e.g., for legends or
@@ -36,17 +43,18 @@
 ## @seealso{hdl2struct, hgload, findobj}
 ## @end deftypefn
 
-## Author: pdiribarne <pdiribarne@new-host.home>
-## Created: 2012-03-04
-
 function [h, pout] = struct2hdl (s, p=[], hilev = false)
 
+  if (nargin < 1)
+    print_usage ();
+  endif
+
   fields = {"handle", "type", "children", "properties", "special"};
   partypes = {"root", "figure", "axes", "hggroup"};
-  othertypes = {"line", "patch", "surface", "image", "text"};
+  othertypes = {"line", "patch", "scatter", "surface", "image", "text"};
   alltypes = [partypes othertypes];
 
-  if (nargin > 3 || ! isstruct (s))
+  if (! isstruct (s))
     print_usage ();
   elseif (! all (isfield (s, fields)))
     print_usage ();
@@ -98,31 +106,47 @@ function [h, pout] = struct2hdl (s, p=[], hilev = false)
 
   ## Use lowercase for all properties
   s.properties = cell2struct (struct2cell (s.properties), ...
-                              tolower (fieldnames (s.properties)));
+                              lower (fieldnames (s.properties)));
 
   ## Place the "*mode" properties at the end to avoid having the updaters
   ## change the mode to "manual" when the value is "auto".
   names = fieldnames (s.properties);
   n = strncmp (cellfun (@fliplr, names, "uniformoutput", false), "edom", 4);
-  n = (n | strcmp (names, "activepositionproperty"));
+  n = (n | strcmp (names, "positionconstraint"));
   names = [names(! n); names(n)];
   n_pos = find (strcmp (names, "position") | strcmp (names, "outerposition"));
   if (strcmp (s.type, "axes") && numel (n_pos) == 2)
-    if (strcmp (s.properties.activepositionproperty, "position"))
-      names{n_pos(1)} = "outerposition";
-      names{n_pos(2)} = "position";
+    if (isfield (s.properties, "positionconstraint"))
+      positionconstraint = s.properties.positionconstraint;
     else
+      ## loading old figure file before "positionconstraint" property was added
+      positionconstraint = s.properties.activepositionproperty;
+    endif
+    if (strcmp (positionconstraint, "outerposition"))
       names{n_pos(1)} = "position";
       names{n_pos(2)} = "outerposition";
+    else
+      names{n_pos(1)} = "outerposition";
+      names{n_pos(2)} = "position";
     endif
   endif
-  ## Reorder the properties with the mode properties coming last
+  ## Set "units" property early
+  n = strcmp (names, "units");
+  names = [names(n); names(! n)];
+  ## Reorder the properties
   s.properties = orderfields (s.properties, names);
 
   ## Silence deprecation warnings
   warning ("off", "Octave:deprecated-property", "local");
 
-  ## create object
+  ## Translate field names for Matlab .fig files
+  ## FIXME: Is it ok to do this unconditionally?
+  if (isfield (s.properties, "applicationdata"))
+    s.properties.__appdata__ = s.properties.applicationdata;
+    s.properties = rmfield (s.properties, "applicationdata");
+  endif
+
+  ## Create object
   if (strcmp (s.type, "root"))
     h = 0;
     s.properties = rmfield (s.properties, ...
@@ -139,21 +163,29 @@ function [h, pout] = struct2hdl (s, p=[], hilev = false)
       if (strcmp (s.properties.tag, "legend"))
         s.properties.tag = "";
         s.properties.userdata = [];
-        par = gcf;
+        par = gcf ();
       elseif (strcmp (s.properties.tag, "colorbar"))
         s.properties.tag = "";
         s.properties.userdata = [];
-        par = gcf;
+        par = gcf ();
       endif
     endif
-    if (isfield (s.properties, "tightinset"))
-      s.properties = rmfield (s.properties, {"tightinset"});
+    ## remove read only properties
+    ## FIXME: Remove "interactions", "layout", "legend", "toolbar", "xaxis",
+    ## "yaxis", and "zaxis" from this list once they are implemented.
+    ro_props = {"interactions", "layout", "legend", "nextseriesindex", ...
+                "tightinset", "toolbar", "xaxis", "yaxis", "zaxis"};
+    has_ro_props = cellfun (@(x) isfield (s.properties, x), ro_props);
+    if (any (has_ro_props))
+      s.properties = rmfield (s.properties, ro_props(has_ro_props));
     endif
     [h, s] = createaxes (s, p, par);
   elseif (strcmp (s.type, "line"))
     h = createline (s, par);
   elseif (strcmp (s.type, "patch"))
     [h, s] = createpatch (s, par);
+  elseif (strcmp (s.type, "scatter"))
+    [h, s] = createscatter (s, par);
   elseif (strcmp (s.type, "text"))
     if (isfield (s.properties, "extent"))
       s.properties = rmfield (s.properties, "extent");
@@ -163,15 +195,21 @@ function [h, pout] = struct2hdl (s, p=[], hilev = false)
     h = createimage (s, par);
   elseif (strcmp (s.type, "surface"))
     h = createsurface (s, par);
+  elseif (strcmp (s.type, "light"))
+    h = createlight (s, par);
   elseif (strcmp (s.type, "hggroup"))
     [h, s, p] = createhg (s, p, par, hilev);
   elseif (any (strcmp (s.type, {"uimenu", "uicontextmenu",...
                                 "uicontrol", "uipanel", "uibuttongroup",...
-                                "uitoolbar", "uipushtool", "uitable"})))
+                                "uitoolbar", "uipushtool", "uitoggletool"...
+                                "uitable"})))
     if (isfield (s.properties, "extent"))
       s.properties = rmfield (s.properties, "extent");
     endif
     [h, s] = createui (s, par);
+    if (strcmp (s.type, "uicontextmenu"))
+      set (p(2, ismember (p(1,:), s.special)), 'UIContextMenu', h);
+    endif
   else
     error ("struct2hdl: %s objects are not implemented yet", s.type);
   endif
@@ -197,6 +235,7 @@ function [h, pout] = struct2hdl (s, p=[], hilev = false)
 endfunction
 
 function [h, sout] = createfigure (s)
+
   ## Create figure initially invisible to speed up loading.
   opts = {"visible", "off"};
   if (isfield (s.properties, "integerhandle"))  # see also bug #53342.
@@ -213,6 +252,7 @@ function [h, sout] = createfigure (s)
   endif
   addmissingprops (h, s.properties);
   sout = s;
+
 endfunction
 
 function [h, sout] = createaxes (s, p, par)
@@ -229,16 +269,17 @@ function [h, sout] = createaxes (s, p, par)
         propval = [propval, prop, val];
       endif
     endfor
-    h = axes (propval{:}, "parent", par);
+    # set hold "on" until all axes children have been added
+    h = axes (propval{:}, "parent", par, "nextplot", "add");
 
     if (isfield (s.properties, "__plotyy_axes__"))
       plty = s.properties.__plotyy_axes__;
       addproperty ("__plotyy_axes__", h, "data");
       tmp = [p [s.handle; h]];
-      tst = ismember (tmp(1:2:end), plty);
+      tst = ismember (tmp(1,:), plty);
       if (sum (tst) == numel (plty))
         for ii = 1:numel (plty)
-          plty(ii) = tmp(find (tmp == plty(ii)) + 1);
+          plty(ii) = tmp(2, find (tmp(1,:) == plty(ii)));
         endfor
         for ii = 1:numel (plty)
           set (plty(ii), "__plotyy_axes__", plty);
@@ -352,6 +393,23 @@ function [h, sout] = createpatch (s, par)
 
 endfunction
 
+function [h, sout] = createscatter (s, par)
+
+  if (isempty (s.properties.zdata))
+    ## 2-D scatter
+    h = scatter (s.properties.xdata, s.properties.ydata, "parent", par);
+  else
+    ## 3-D scatter
+    h = scatter3 (s.properties.xdata, s.properties.ydata, s.properties.zdata, "parent", par);
+  endif
+
+  s.properties = rmfield (s.properties,
+                          {"xdata", "ydata", "zdata"});
+  addmissingprops (h, s.properties);
+  sout = s;
+
+endfunction
+
 function h = createtext (s, par)
   h = text ("parent", par);
   addmissingprops (h, s.properties);
@@ -367,6 +425,11 @@ function h = createsurface (s, par)
   addmissingprops (h, s.properties);
 endfunction
 
+function h = createlight (s, par)
+  h = light ("parent", par);
+  addmissingprops (h, s.properties);
+endfunction
+
 function [h, s] = createui (s, par)
   if (isfield (s.properties, "style") && strcmp (s.properties.style, "frame"))
     s.type = "uipanel";  # frame is deprecated: use uipanel instead
@@ -376,10 +439,11 @@ function [h, s] = createui (s, par)
 endfunction
 
 function [h, sout, pout] = createhg (s, p, par, hilev)
+
   ## Here we infer from properties the type of hggroup we should build
-  ## an call corresponding high level functions
+  ## and call the corresponding high level functions.
   ## We manually set "hold on" to avoid next hggroup be deleted
-  ## the proper value of axes "nextplot" will finally be recovered
+  ## the proper value of axes "nextplot" will finally be recovered.
 
   hold on;
   if (hilev)
@@ -411,6 +475,12 @@ function [h, sout, pout] = createhg (s, p, par, hilev)
     s.special = [];
   endif
 
+  ## Check for polar plots with special "polar_grid" object
+  ## FIXME: A hack to fix bug #62093.
+  if (strcmp (s.properties.tag, "polar_grid"))
+    s.properties.handlevisibility = "off";
+  endif
+
   sout = s;
   pout = p;
 
@@ -472,7 +542,7 @@ function [h, sout, pout] = createhg_hilev (s, p, par)
                                "udatasource", "vdatasource", "wdatasource"});
 
   elseif (isfield (fields, "format"))
-    ##errorbar
+    ## errorbar
     form = s.properties.format;
     xdata = s.properties.xdata;
     ydata = s.properties.ydata;
@@ -536,7 +606,7 @@ function [h, sout, pout] = createhg_hilev (s, p, par)
       xdata = [];
       ydata = [];
 
-      ##build x/y matrix
+      ## Build x/y matrix
       nbar = length (bargroup);
       tmp = struct ("handle", NaN, "type", "", "children", [], "special", []);
       for ii = 1:(nbar - 1)
@@ -719,4 +789,25 @@ function addmissingprops (h, props)
 endfunction
 
 
-## FIXME: Need validation tests
+## Check that all axes children are restored
+%!test <*66221>
+%! hf = figure ("visible", "off");
+%! unwind_protect
+%!   hax = axes (hf);
+%!   hold (hax, "on");
+%!   imagesc (hax, randn (5, 3));
+%!   scatter (hax, 1:3, rand (3, 1));
+%!   hold (hax, "off");
+%!   n_children = numel (get (hax, "children"));
+%!   tmp_ofig = [tempname(), ".ofig"];
+%!   s = hdl2struct (hf);
+%! unwind_protect_cleanup
+%!   close (hf);
+%! end_unwind_protect
+%! hf_new = struct2hdl (s);
+%! unwind_protect
+%!   hax_new = findobj (hf_new, "type", "axes");
+%!   assert (numel (get (hax_new, "children")), n_children);
+%! unwind_protect_cleanup
+%!   close (hf_new);
+%! end_unwind_protect
