@@ -1,6 +1,6 @@
 ########################################################################
 ##
-## Copyright (C) 2018-2023 The Octave Project Developers
+## Copyright (C) 2018-2024 The Octave Project Developers
 ##
 ## See the file COPYRIGHT.md in the top-level directory of this
 ## distribution or <https://octave.org/copyright/>.
@@ -34,14 +34,14 @@
 ##
 ## If @var{wlen} is a scalar, the function @code{max} is applied to a
 ## moving window of length @var{wlen}.  When @var{wlen} is an odd number the
-## window is symmetric and includes @w{@code{(@var{wlen} - 1) / 2}} elements on
-## either side of the central element.  For example, when calculating the
+## window is symmetric and includes @w{@code{(@var{wlen} - 1) / 2}}@ elements
+## on either side of the central element.  For example, when calculating the
 ## output at index 5 with a window length of 3, @code{movmax} uses data
 ## elements @w{@code{[4, 5, 6]}}.  If @var{wlen} is an even number, the window
-## is asymmetric and has @w{@code{@var{wlen}/2}} elements to the left of the
-## central element and @w{@code{@var{wlen}/2 - 1}} elements to the right of the
-## central element.  For example, when calculating the output at index 5 with a
-## window length of 4, @code{movmax} uses data elements
+## is asymmetric and has @w{@code{@var{wlen}/2}}@ elements to the left of the
+## central element and @w{@code{@var{wlen}/2 - 1}}@ elements to the right of
+## the central element.  For example, when calculating the output at index 5
+## with a window length of 4, @code{movmax} uses data elements
 ## @w{@code{[3, 4, 5, 6]}}.
 ##
 ## If @var{wlen} is an array with two elements @w{@code{[@var{nb}, @var{na}]}},
@@ -82,8 +82,8 @@
 ## data array are deleted.  For example, with a 10-element data vector and a
 ## window of length 3, the output will contain only 8 elements.  The first
 ## element would require calculating the function over indices
-## @w{@code{[0, 1, 2]}} and is therefore discarded.  The last element would
-## require calculating the function over indices @w{@code{[9, 10, 11]}} and is
+## @w{@code{[0, 1, 2]}}@ and is therefore discarded.  The last element would
+## require calculating the function over indices @w{@code{[9, 10, 11]}}@ and is
 ## therefore discarded.
 ##
 ## @item @qcode{"fill"}
@@ -141,9 +141,18 @@ function y = movmax (x, wlen, varargin)
 endfunction
 
 
-## FIXME: Need functional BIST tests
-## test for bug #55241
-%!assert ([(2:10).'; 10], movmax ((1:10).', 3))
+%!assert (movmax (1:5, 3), [2:5, 5])
+%!assert (movmax (1:5, [1, 1]), [2:5, 5])
+%!assert (movmax (1:5, 3, 2), [2:5, 5])
+
+%!assert (movmax (magic (4), 3),
+%!        [16, 11, 10, 13; 16, 11, 10, 13; 9, 14, 15, 12; 9, 14, 15, 12])
+%!assert (movmax (magic (4), 3, 1),
+%!        [16, 11, 10, 13; 16, 11, 10, 13; 9, 14, 15, 12; 9, 14, 15, 12])
+%!assert (movmax (magic (4), 3, 2),
+%!        [16, 16, 13, 13; 11, 11, 11, 10; 9, 9, 12, 12; 14, 15, 15, 15])
+
+%!assert <*55241> (movmax ((1:10).', 3), [(2:10).'; 10])
 
 ## Test input validation
 %!error <Invalid call> movmax ()
