@@ -1,15 +1,15 @@
 ////////////////////////////////////////////////////////////////////////
 //
-// Copyright (C) 2017-2021 The Octave Project Developers
+// Copyright (C) 2017-2024 The Octave Project Developers
 //
 // See the file COPYRIGHT.md in the top-level directory of this
 // distribution or <https://octave.org/copyright/>.
 //
 // This file is part of Octave.
 //
-// Octave is free software; you can redistribute it and/or modify it
+// Octave is free software: you can redistribute it and/or modify it
 // under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 3 of the License, or
+// the Free Software Foundation, either version 3 of the License, or
 // (at your option) any later version.
 //
 // Octave is distributed in the hope that it will be useful, but
@@ -19,12 +19,23 @@
 //
 // You should have received a copy of the GNU General Public License
 // along with Octave; see the file COPYING.  If not, see
-// <http://www.gnu.org/licenses/>.
+// <https://www.gnu.org/licenses/>.
 //
 ////////////////////////////////////////////////////////////////////////
 
+#if defined (HAVE_CONFIG_H)
+#  include "config.h"
+#endif
+
+#include <cstdio>
 #include <iostream>
 
+#if defined (OCTAVE_USE_WINDOWS_API)
+#  include <vector>
+#  include <locale>
+#  include <codecvt>
+#endif
+
 #include <QtCore>
 #include <QtXml>
 
@@ -33,77 +44,71 @@
 #include <QImage>
 #include <QPainter>
 #include <QPrinter>
-#include <QRegExp>
+#include <QRegularExpression>
 
+// Include a set of path rendering functions extracted from Qt-5.12 source
+#include "octave-qsvghandler.h"
+
+// Render to pdf
 class pdfpainter : public QPainter
 {
 public:
-  pdfpainter (QString fname, QRectF sizepix, double dpi)
-    : m_fname (fname), m_sizef (sizepix), m_dpi (dpi), m_printer ()
-  {
-    double scl = get_scale ();
-    m_sizef.setWidth (m_sizef.width () * scl);
-    m_sizef.setHeight (m_sizef.height () * scl);
 
+  pdfpainter () = delete;
+
+  pdfpainter (QString fname, QRectF sz)
+    : m_printer ()
+  {
     // Printer settings
     m_printer.setOutputFormat (QPrinter::PdfFormat);
     m_printer.setFontEmbeddingEnabled (true);
-    m_printer.setOutputFileName (get_fname ());
+    m_printer.setOutputFileName (fname);
     m_printer.setFullPage (true);
-    m_printer.setPaperSize (get_rectf ().size (), QPrinter::DevicePixel);
+    m_printer.setPageSize (QPageSize (sz.size (), QPageSize::Point,
+                                      QString ("custom"),
+                                      QPageSize::ExactMatch));
 
     // Painter settings
     begin (&m_printer);
-    setViewport (get_rect ());
-    scale (get_scale (), get_scale ());
+    setWindow (sz.toRect ());
   }
 
-  ~pdfpainter (void) { }
+  OCTAVE_DISABLE_COPY_MOVE (pdfpainter)
 
-  QString get_fname (void) const { return m_fname; }
-
-  QRectF get_rectf (void) const { return m_sizef; }
-
-  QRect get_rect (void) const { return m_sizef.toRect (); }
-
-  double get_scale (void) const { return m_dpi / 72.0; }
-
-  void finish (void) { end (); }
+  ~pdfpainter () { end (); }
 
 private:
-  QString m_fname;
-  QRectF m_sizef;
-  double m_dpi;
+
   QPrinter m_printer;
 };
 
-// String conversion functions
-QVector<double> qstr2vectorf (QString str)
+// String conversion functions+QVector<double> qstr2vectorf (QString str)
+QVector<double>
+qstr2vectorf (QString str)
 {
   QVector<double> pts;
   QStringList coords = str.split (",");
-  for (QStringList::iterator p = coords.begin (); p != coords.end (); p += 1)
-    {
-      double pt = (*p).toDouble ();
-      pts.append (pt);
-    }
-  return pts;
-}
-
-QVector<double> qstr2vectord (QString str)
-{
-  QVector<double> pts;
-  QStringList coords = str.split (",");
-  for (QStringList::iterator p = coords.begin (); p != coords.end (); p += 1)
-    {
-      double pt = (*p).toDouble ();
-      pts.append (pt);
-    }
+  for (auto& p : coords)
+    pts.append (p.toDouble ());
 
   return pts;
 }
 
-QVector<QPointF> qstr2ptsvector (QString str)
+// FIXME: What's the difference between qstr2vectorf and qstr2vectord?
+// Can one be called from the other to avoid code duplication, or deleted?
+QVector<double>
+qstr2vectord (QString str)
+{
+  QVector<double> pts;
+  QStringList coords = str.split (",");
+  for (auto& p : coords)
+    pts.append (p.toDouble ());
+
+  return pts;
+}
+
+QVector<QPointF>
+qstr2ptsvector (QString str)
 {
   QVector<QPointF> pts;
   str = str.trimmed ();
@@ -117,7 +122,10 @@ QVector<QPointF> qstr2ptsvector (QString str)
   return pts;
 }
 
-QVector<QPoint> qstr2ptsvectord (QString str)
+// FIXME: What's the difference between qstr2ptsvector and qstr2ptsvectord?
+// Can one be called from the other to avoid code duplication, or deleted?
+QVector<QPoint>
+qstr2ptsvectord (QString str)
 {
   QVector<QPoint> pts;
   str = str.trimmed ();
@@ -132,14 +140,12 @@ QVector<QPoint> qstr2ptsvectord (QString str)
 }
 
 // Extract field arguments in a style-like string, e.g. "bla field(1,34,56) bla"
-QString get_field (QString str, QString field)
+QString
+get_field (QString str, QString field)
 {
-  QString retval;
-  QRegExp rx (field + "\\(([^\\)]*)\\)");
-  int pos = 0;
-  pos = rx.indexIn (str, pos);
-  if (pos > -1)
-    retval = rx.cap (1);
+  QRegularExpression rx (field + "\\(([^\\)]*)\\)");
+  QRegularExpressionMatch match = rx.match (str);
+  QString retval = match.captured (1);
 
   return retval;
 }
@@ -148,29 +154,27 @@ QString get_field (QString str, QString field)
 class octave_polygon
 {
 public:
-  octave_polygon (void)
-  { }
 
   octave_polygon (QPolygonF p)
   { m_polygons.push_back (p); }
 
-  ~octave_polygon (void) { }
+  OCTAVE_DEFAULT_CONSTRUCT_COPY_MOVE_DELETE (octave_polygon)
 
-  int count (void) const
+  int count () const
   { return m_polygons.count (); }
 
-  void reset (void)
+  void reset ()
   { m_polygons.clear (); }
 
-  QList<QPolygonF> reconstruct (void)
+  QList<QPolygonF> reconstruct (int reconstruct_level)
   {
     if (m_polygons.isEmpty ())
       return QList<QPolygonF> ();
+    else if (reconstruct_level < 2)
+      return m_polygons;
 
     // Once a polygon has been merged to another, it is marked unsuded
-    QVector<bool> unused;
-    for (auto it = m_polygons.begin (); it != m_polygons.end (); it++)
-      unused.push_back (false);
+    QVector<bool> unused (m_polygons.count (), false);
 
     bool tryagain = (m_polygons.count () > 1);
 
@@ -332,18 +336,23 @@ private:
   QList<QPolygonF> m_polygons;
 };
 
-void draw (QDomElement& parent_elt, pdfpainter& painter)
+void
+draw (QDomElement& parent_elt, pdfpainter& painter)
 {
   QDomNodeList nodes = parent_elt.childNodes ();
 
   static QString clippath_id;
-  static QMap< QString, QVector<QPoint> > clippath;
+  static QMap< QString, QVector<QPoint>> clippath;
 
   // tspan elements must have access to the font and position extracted from
   // their parent text element
   static QFont font;
   static double dx = 0, dy = 0;
 
+  // Store path defined in <defs> in a map
+  static bool in_defs = false;
+  static QMap< QString, QPainterPath> path_map;
+
   for (int i = 0; i < nodes.count (); i++)
     {
       QDomNode node = nodes.at (i);
@@ -360,86 +369,179 @@ void draw (QDomElement& parent_elt, pdfpainter& painter)
         }
       else if (elt.tagName () == "g")
         {
-          bool current_clipstate = painter.hasClipping ();
-          QRegion current_clippath = painter.clipRegion ();
-
-          QString str = elt.attribute ("clip-path");
-          if (! str.isEmpty ())
-            {
-              QVector<QPoint> pts = clippath[get_field (str, "url")];
-              if (! pts.isEmpty ())
-                {
-                  painter.setClipRegion (QRegion (QPolygon (pts)));
-                  painter.setClipping (true);
-                }
-            }
-
-          draw (elt, painter);
-
-          // Restore previous clipping settings
-          painter.setClipRegion (current_clippath);
-          painter.setClipping (current_clipstate);
-        }
-      else if (elt.tagName () == "text")
-        {
-          // Font
-          font = QFont ();
           QString str = elt.attribute ("font-family");
-          if (! str.isEmpty ())
-            font.setFamily (elt.attribute ("font-family"));
-
-          str = elt.attribute ("font-weight");
-          if (! str.isEmpty () && str != "normal")
-            font.setWeight (QFont::Bold);
-
-          str = elt.attribute ("font-style");
-          if (! str.isEmpty () && str != "normal")
-            font.setStyle (QFont::StyleItalic);
-
-          str = elt.attribute ("font-size");
-          if (! str.isEmpty ())
-            font.setPixelSize (str.toDouble ());
-
-          painter.setFont (font);
-
-          // Translation and rotation
-          painter.save ();
-          str = get_field (elt.attribute ("transform"), "translate");
           if (! str.isEmpty ())
             {
-              QStringList trans = str.split (",");
-              dx = trans[0].toDouble ();
-              dy = trans[1].toDouble ();
+              // Font
+              font = QFont (str);
 
-              str = get_field (elt.attribute ("transform"), "rotate");
+              str = elt.attribute ("font-weight");
+              if (! str.isEmpty () && str != "normal")
+                font.setWeight (QFont::Bold);
+
+              str = elt.attribute ("font-style");
+              if (! str.isEmpty () && str != "normal")
+                font.setStyle (QFont::StyleItalic);
+
+              str = elt.attribute ("font-size");
+              if (! str.isEmpty ())
+                font.setPixelSize (str.toDouble ());
+
+              painter.setFont (font);
+
+              // Translation and rotation
+              painter.save ();
+              str = get_field (elt.attribute ("transform"), "translate");
               if (! str.isEmpty ())
                 {
-                  QStringList rot = str.split (",");
-                  painter.translate (dx+rot[1].toDouble (),
-                                     dy+rot[2].toDouble ());
-                  painter.rotate (rot[0].toDouble ());
-                  dx = rot[1].toDouble ();
-                  dy = rot[2].toDouble ();
+                  QStringList trans = str.split (",");
+                  dx = trans[0].toDouble ();
+                  dy = trans[1].toDouble ();
+
+                  str = get_field (elt.attribute ("transform"), "rotate");
+                  if (! str.isEmpty ())
+                    {
+                      QStringList rot = str.split (",");
+                      painter.translate (dx+rot[1].toDouble (),
+                                         dy+rot[2].toDouble ());
+                      painter.rotate (rot[0].toDouble ());
+                      dx = rot[1].toDouble ();
+                      dy = rot[2].toDouble ();
+                    }
+                  else
+                    {
+                      painter.translate (dx, dy);
+                      dx = 0;
+                      dy = 0;
+                    }
                 }
-              else
+
+              draw (elt, painter);
+              painter.restore ();
+            }
+          else
+            {
+              bool current_clipstate = painter.hasClipping ();
+              QRegion current_clippath = painter.clipRegion ();
+
+              str = elt.attribute ("clip-path");
+              if (! str.isEmpty ())
                 {
-                  painter.translate (dx, dy);
-                  dx = 0;
-                  dy = 0;
+                  QVector<QPoint> pts = clippath[get_field (str, "url")];
+                  if (! pts.isEmpty ())
+                    {
+                      painter.setClipRegion (QRegion (QPolygon (pts)));
+                      painter.setClipping (true);
+                    }
+                }
+
+              // Fill color
+              str = get_field (elt.attribute ("fill"), "rgb");
+              if (! str.isEmpty ())
+                {
+                  QStringList clist = str.split (",");
+                  painter.setBrush (QColor (clist[0].toInt (),
+                                            clist[1].toInt (),
+                                            clist[2].toInt ()));
+                }
+
+              // Transform
+              str = elt.attribute ("transform");
+              painter.save ();
+              if (! str.isEmpty ())
+                {
+#if HAVE_QSTRINGVIEW
+                  QStringView tf {str};
+#else
+                  QStringRef tf {&str};
+#endif
+                  QTransform  tform =
+                    parseTransformationMatrix (tf) * painter.transform ();
+                  painter.setTransform (tform);
+                }
+
+              draw (elt, painter);
+
+              // Restore previous clipping settings
+              painter.restore  ();
+              painter.setClipRegion (current_clippath);
+              painter.setClipping (current_clipstate);
+            }
+        }
+      else if (elt.tagName () == "defs")
+        {
+          in_defs = true;
+          draw (elt, painter);
+          in_defs = false;
+        }
+      else if (elt.tagName () == "path")
+        {
+          // Store QPainterPath for latter use
+          QString id = elt.attribute ("id");
+          if (! id.isEmpty ())
+            {
+              QString d = elt.attribute ("d");
+
+              if (! d.isEmpty ())
+                {
+#if HAVE_QSTRINGVIEW
+                  QStringView data {d};
+#else
+                  QStringRef data {&d};
+#endif
+                  QPainterPath path;
+                  if (! parsePathDataFast (data, path))
+                    continue; // Something went wrong, pass
+                  else if (path.isEmpty ())
+                    std::cout << "Empty path for data:"
+                              << d.toStdString () << std::endl;
+                  else if (in_defs)
+                    path_map["#" + id] = path;
+                  else
+                    painter.drawPath (path);
+
+
+                  if (path_map["#" + id].isEmpty ())
+                    std::cout << "Empty path for data:"
+                              << d.toStdString () << std::endl;
                 }
             }
-
-          draw (elt, painter);
-          painter.restore ();
         }
-      else if (elt.tagName () == "tspan")
+      else if (elt.tagName () == "use")
+        {
+          painter.setPen (Qt::NoPen);
+
+          QString str = elt.attribute ("xlink:href");
+          if (! str.isEmpty () && str.size () > 2)
+            {
+              QPainterPath path = path_map[str];
+              if (! path.isEmpty ())
+                {
+                  str = elt.attribute ("x");
+                  double x = elt.attribute ("x").toDouble ();
+                  str = elt.attribute ("y");
+                  double y = elt.attribute ("y").toDouble ();
+                  painter.translate (x, y);
+                  painter.drawPath (path);
+                  painter.translate (-x, -y);
+                }
+            }
+        }
+      else if (elt.tagName () == "text")
         {
           // Font
           QFont saved_font (font);
 
           QString str = elt.attribute ("font-family");
           if (! str.isEmpty ())
-            font.setFamily (elt.attribute ("font-family"));
+            {
+              // QFont::setFamily() doesn't seem to work properly in Qt6
+              // (see bug #66306). Use the QFont constructor to update
+              // the current font.
+              font = QFont (str, -1, font.weight (),
+                            font.style () ==  QFont::StyleItalic);
+              font.setPixelSize (saved_font.pixelSize ());
+            }
 
           str = elt.attribute ("font-weight");
           if (! str.isEmpty ())
@@ -504,9 +606,9 @@ void draw (QDomElement& parent_elt, pdfpainter& painter)
           str = elt.attribute ("stroke-width");
           if (! str.isEmpty ())
             {
-              double w = str.toDouble () * painter.get_scale ();
+              double w = str.toDouble ();
               if (w > 0)
-                pen.setWidthF (w / painter.get_scale ());
+                pen.setWidthF (w);
             }
 
           str = elt.attribute ("stroke-linecap");
@@ -562,18 +664,44 @@ void draw (QDomElement& parent_elt, pdfpainter& painter)
               if (! str.isEmpty ())
                 {
                   QVector<double> m = qstr2vectorf (str);
-                  double scl = painter.get_scale ();
-                  QTransform tform(m[0]*scl, m[1]*scl, m[2]*scl,
-                                   m[3]*scl, m[4]*scl, m[5]*scl);
+                  QTransform tform(m[0], m[1], m[2],
+                                   m[3], m[4], m[5]);
                   painter.setTransform (tform);
                 }
 
               painter.setRenderHint (QPainter::Antialiasing, false);
+#if defined (HAVE_QPAINTER_RENDERHINT_LOSSLESS)
+              painter.setRenderHint (QPainter::LosslessImageRendering);
+#endif
               painter.drawImage (pos, img);
               painter.setRenderHint (QPainter::Antialiasing, true);
               painter.restore  ();
             }
         }
+      else if (elt.tagName () == "rect")
+        {
+          // Position
+          double x = elt.attribute ("x").toDouble ();
+          double y = elt.attribute ("y").toDouble ();
+
+          // Size
+          double wd = elt.attribute ("width").toDouble ();
+          double hg = elt.attribute ("height").toDouble ();
+
+          // Color
+          QColor saved_color = painter.brush ().color ();
+
+          QString str = elt.attribute ("fill");
+          if (! str.isEmpty ())
+            painter.setBrush (QColor (str));
+
+          painter.setPen (Qt::NoPen);
+
+          painter.drawRect (QRectF (x, y, wd, hg));
+
+          if (! str.isEmpty ())
+            painter.setBrush (saved_color);
+        }
       else if (elt.tagName () == "polygon")
         {
           if (! clippath_id.isEmpty ())
@@ -610,8 +738,9 @@ void draw (QDomElement& parent_elt, pdfpainter& painter)
 // Append a list of reconstructed child polygons to a QDomElement and remove
 // the original nodes
 
-void replace_polygons (QDomElement& parent_elt, QList<QDomNode> orig,
-                       QList<QPolygonF> polygons)
+void
+replace_polygons (QDomElement& parent_elt, QList<QDomNode> orig,
+                  QList<QPolygonF> polygons)
 {
   if (! orig.count () || (orig.count () == polygons.count ()))
     return;
@@ -641,7 +770,8 @@ void replace_polygons (QDomElement& parent_elt, QList<QDomNode> orig,
     parent_elt.removeChild (orig.at (ii));
 }
 
-void reconstruct_polygons (QDomElement& parent_elt)
+void
+reconstruct_polygons (QDomElement& parent_elt, int reconstruct_level)
 {
   QDomNodeList nodes = parent_elt.childNodes ();
   QColor current_color;
@@ -649,7 +779,7 @@ void reconstruct_polygons (QDomElement& parent_elt)
   octave_polygon current_polygon;
 
   // Collection of child nodes to be removed and polygons to be added
-  QList< QPair<QList<QDomNode>,QList<QPolygonF> > > collection;
+  QList< QPair<QList<QDomNode>, QList<QPolygonF>>> collection;
 
   for (int ii = 0; ii < nodes.count (); ii++)
     {
@@ -669,7 +799,7 @@ void reconstruct_polygons (QDomElement& parent_elt)
               if (! str.isEmpty ())
                 {
                   double alpha = str.toDouble ();
-                  if (alpha != 1.0 && str.toDouble () >= 0.0)
+                  if (alpha != 1.0 && alpha >= 0.0)
                     color.setAlphaF (alpha);
                 }
 
@@ -678,9 +808,10 @@ void reconstruct_polygons (QDomElement& parent_elt)
 
               if (color != current_color)
                 {
-                  // Reconstruct the previous series of triangle
-                  QList<QPolygonF> polygons = current_polygon.reconstruct ();
-                  collection.push_back (QPair<QList<QDomNode>,QList<QPolygonF> >
+                  // Reconstruct the previous series of triangles
+                  QList<QPolygonF> polygons
+                    = current_polygon.reconstruct (reconstruct_level);
+                  collection.push_back (QPair<QList<QDomNode>, QList<QPolygonF>>
                                         (replaced_nodes, polygons));
 
                   replaced_nodes.clear ();
@@ -698,26 +829,69 @@ void reconstruct_polygons (QDomElement& parent_elt)
         {
           if (current_polygon.count ())
             {
-              QList<QPolygonF> polygons = current_polygon.reconstruct ();
-              collection.push_back (QPair<QList<QDomNode>,QList<QPolygonF> >
+              QList<QPolygonF> polygons = current_polygon.reconstruct (reconstruct_level);
+              collection.push_back (QPair<QList<QDomNode>, QList<QPolygonF>>
                                     (replaced_nodes, polygons));
               replaced_nodes.clear ();
               current_polygon.reset ();
             }
-          reconstruct_polygons (elt);
+          reconstruct_polygons (elt, reconstruct_level);
         }
     }
 
   // Finish
-  collection.push_back (QPair<QList<QDomNode>,QList<QPolygonF> >
-                        (replaced_nodes, current_polygon.reconstruct ()));
+  collection.push_back (QPair<QList<QDomNode>, QList<QPolygonF>>
+                        (replaced_nodes,
+                         current_polygon.reconstruct (reconstruct_level)));
 
   for (int ii = 0; ii < collection.count (); ii++)
     replace_polygons (parent_elt, collection[ii].first, collection[ii].second);
 }
 
-int main(int argc, char *argv[])
+void
+add_custom_properties (QDomElement& parent_elt)
 {
+  QDomNodeList nodes = parent_elt.childNodes ();
+
+  for (int ii = 0; ii < nodes.count (); ii++)
+    {
+      QDomNode node = nodes.at (ii);
+      if (! node.isElement ())
+        continue;
+
+      QDomElement elt = node.toElement ();
+
+      if (elt.tagName () == "image")
+        elt.setAttribute ("image-rendering", "optimizeSpeed");
+      else
+        add_custom_properties (elt);
+    }
+
+}
+
+#if defined (OCTAVE_USE_WINDOWS_API) && defined (_UNICODE)
+extern "C"
+int
+wmain (int argc, wchar_t **wargv)
+{
+  static char **argv = new char * [argc + 1];
+  std::vector<std::string> argv_str;
+
+  // convert wide character strings to multibyte UTF-8 strings
+  std::wstring_convert<std::codecvt_utf8<wchar_t>, wchar_t> wchar_conv;
+  for (int i_arg = 0; i_arg < argc; i_arg++)
+    argv_str.push_back (wchar_conv.to_bytes (wargv[i_arg]));
+
+  // Get pointers to C strings not before vector is stable.
+  for (int i_arg = 0; i_arg < argc; i_arg++)
+    argv[i_arg] = &argv_str[i_arg][0];
+  argv[argc] = nullptr;
+
+#else
+int
+main (int argc, char **argv)
+{
+#endif
   const char *doc = "See \"octave-svgconvert -h\"";
   const char *help = "Usage:\n\
 octave-svgconvert infile fmt dpi font reconstruct outfile\n\n\
@@ -727,7 +901,10 @@ read from stdin\n\
 * fmt: format of the output file. May be one of pdf or svg\n\
 * dpi: device dependent resolution in screen pixel per inch\n\
 * font: specify a file name for the default FreeSans font\n\
-* reconstruct: specify whether to reconstruct triangle to polygons (0 or 1)\n\
+* reconstruct: specify whether to reconstruct triangle to polygons\n\
+  0: no reconstruction (merging) of polygons\n\
+  1: merge consecutive triangles if they share an edge\n\
+  2: merge all triangles that share edges (might take a long time)\n\
 * outfile: output file name\n";
 
   if (strcmp (argv[1], "-h") == 0)
@@ -759,7 +936,7 @@ read from stdin\n\
       // Read from stdin
       if (! file.open (stdin, QIODevice::ReadOnly | QIODevice::Text))
         {
-          std::cerr << "Unable read from stdin\n";
+          std::cerr << "Unable to read from stdin\n";
           std::cerr << doc;
           return -1;
         }
@@ -771,8 +948,7 @@ read from stdin\n\
   if (! document.setContent (&file, false, &msg))
     {
       std::cerr << "Failed to parse XML contents" << std::endl
-                << msg.toStdString ();
-      std::cerr << doc;
+                << msg.toStdString () << std::endl;
       file.close();
       return -1;
     }
@@ -787,14 +963,9 @@ read from stdin\n\
       return -1;
     }
 
-  // Resolution
-  double dpi = QString (argv[3]).toDouble ();
-  if (dpi <= 0.0)
-    {
-      std::cerr << "DPI must be positive\n";
-      return -1;
-    }
-
+  // Resolution (Currently unused). Keep the DPI argument in case
+  // we implement raster outputs.
+  // double dpi = QString (argv[3]).toDouble ();
 
   // Get the viewport from the root element
   QDomElement root = document.firstChildElement();
@@ -834,23 +1005,30 @@ read from stdin\n\
     }
 
   // Do basic polygons reconstruction
-  if (QString (argv[5]).toInt ())
-    reconstruct_polygons (root);
+  int reconstruct_level = QString (argv[5]).toInt ();
+  if (reconstruct_level)
+    reconstruct_polygons (root, reconstruct_level);
+
+  // Add custom properties to SVG
+  add_custom_properties (root);
 
   // Draw
   if (! strcmp (argv[2], "pdf"))
     {
       // PDF painter
-      pdfpainter painter (fout.fileName (), vp, dpi);
+      pdfpainter painter (fout.fileName (), vp);
 
       draw (root, painter);
-      painter.finish ();
     }
   else
     {
       // Return modified svg document
       QTextStream out (&fout);
+#if HAVE_QTEXTSTREAM_SETENCODING
+      out.setEncoding (QStringConverter::Utf8);
+#else
       out.setCodec ("UTF-8");
+#endif
       out << document.toByteArray ();
     }
 
